@@ -1,16 +1,8 @@
-use std::error::Error;
-
+#[macro_use]
 extern crate rmididings;
 use rmididings::*;
 
-fn main() {
-    match run() {
-        Ok(_) => (),
-        Err(err) => println!("Error: {}", err)
-    }
-}
-
-fn run() -> Result<(), Box<dyn Error>> {
+fn main() -> Result<(), Box<dyn std::error::Error>> {
 
     let mut md = RMididings::new()?;
 
@@ -39,12 +31,11 @@ fn run() -> Result<(), Box<dyn Error>> {
             }
         ],
         control: &Fork!(
-            Chain!(KeyFilter(62), SceneSwitch(2), Discard()),
-            Chain!(KeyFilter(60), SceneSwitch(1), Discard())
+            Chain!(TypeFilter!(Note), KeyFilter(62), SceneSwitch(2)),
+            Chain!(TypeFilter!(Note), KeyFilter(60), SceneSwitch(1))
         ),
         ..RunArguments::default()
     })?;
 
     Ok(())
 }
-
