@@ -37,16 +37,4 @@ impl Scene<'_> {
             None
         }
     }
-}
-
-impl<'a> From<&'a dyn FilterTrait> for Scene<'a> {
-    fn from(ft: &'a dyn FilterTrait) -> Scene<'a> {
-        Scene {
-            name: "Single patch",
-            patch: ft,
-            init: &Discard(),
-            exit: &Discard(),
-            subscenes: &[],
-        }
-    }
-}
+}
\ No newline at end of file
